dnl Convenience macros
dnl last updated from template-1.0.4

dnl Configure-time switch with default
dnl
dnl Each switch defines an --enable-FOO and --disable-FOO option in
dnl the resulting configure script.
dnl
dnl Usage:
dnl smr_SWITCH(name, description, default, pos-def, neg-def)
dnl
dnl where:
dnl
dnl name        name of switch; generates --enable-name & --disable-name
dnl             options
dnl description help string is set to this prefixed by "enable" or
dnl             "disable", whichever is the non-default value
dnl default     either "on" or "off"; specifies default if neither
dnl             --enable-name nor --disable-name is specified
dnl pos-def     a symbol to AC_DEFINE if switch is on (optional)
dnl neg-def     a symbol to AC_DEFINE if switch is off (optional)
dnl
AC_DEFUN(smr_SWITCH, [
    AC_MSG_CHECKING(whether to enable [$2])
    AC_ARG_ENABLE(
        $1,
	ifelse($3, on,
		[AC_HELP_STRING([--disable-$1], [disable $2])],
		[AC_HELP_STRING([--enable-$1], [enable $2])]
	),
        [ if test "$enableval" = yes; then
            AC_MSG_RESULT(yes)
            ifelse($4, , , [AC_DEFINE($4)])
        else
            AC_MSG_RESULT(no)
            ifelse($5, , , [AC_DEFINE($5)])
        fi ],
        ifelse($3, on,
           [ AC_MSG_RESULT(yes)
            ifelse($4, , , [AC_DEFINE($4)]) ],
           [ AC_MSG_RESULT(no)
            ifelse($5, , , [AC_DEFINE($5)])])
	)
])

dnl Allow argument for optional libraries; wraps AC_ARG_WITH, to
dnl provide a "--with-foo-library" option in the configure script, where foo
dnl is presumed to be a library name.  The argument given by the user
dnl (i.e. "bar" in ./configure --with-foo-library=bar) may be one of three
dnl things:
dnl     * boolean (no, yes or blank): whether to use library or not
dnl     * file: assumed to be the name of the library
dnl     * directory: assumed to *contain* the library
dnl
dnl The argument is sanity-checked.  If all is well, two variables are
dnl set: "with_foo" (value is yes, no, or maybe), and "foo_LIBS" (value
dnl is either blank, a file, -lfoo, or '-L/some/dir -lfoo').  The idea
dnl is: the first tells you whether the library is to be used or not
dnl (or the user didn't specify one way or the other) and the second
dnl to put on the command line for linking with the library.
dnl
dnl Usage:
dnl smr_ARG_WITHLIB(name, libname, description)
dnl
dnl name                name for --with argument ("foo" for libfoo)
dnl libname             (optional) actual name of library,
dnl                     if different from name
dnl description         (optional) used to construct help string
dnl
AC_DEFUN(smr_ARG_WITHLIB, [

ifelse($2, , smr_lib=[$1], smr_lib=[$2])

AC_ARG_WITH([$1]-library,
ifelse($3, ,

[AC_HELP_STRING([--with-$1-library[=PATH]], [use $1 library])],
[AC_HELP_STRING([--with-$1-library[=PATH]], [use $1 library ($3)])]),
[
    if test "$withval" = yes; then
        with_[$1]=yes
        [$1]_LIBS="-l${smr_lib}"
    elif test "$withval" = no; then
        with_[$1]=no
        [$1]_LIBS=
    else
        with_[$1]=yes
        if test -f "$withval"; then
            [$1]_LIBS=$withval
        elif test -d "$withval"; then
            [$1]_LIBS="-L$withval -l${smr_lib}"
        else
            AC_MSG_ERROR(argument must be boolean, file, or directory)
        fi
    fi
], [
    with_[$1]=maybe
    [$1]_LIBS="-l${smr_lib}"
])
withval_lib="$withval"
])


dnl Check if the include files for a library are accessible, and
dnl define the variable "name_CFLAGS" with the proper "-I" flag for
dnl the compiler.  The user has a chance to specify the includes
dnl location, using "--with-foo-includes".
dnl
dnl This should be used *after* smr_ARG_WITHLIB *and* AC_CHECK_LIB are
dnl successful.
dnl
dnl Usage:
dnl smr_ARG_WITHINCLUDES(name, header, extra-flags)
dnl
dnl name                library name, MUST same as used with smr_ARG_WITHLIB
dnl header              a header file required for using the lib
dnl extra-flags         (optional) flags required when compiling the
dnl                     header, typically more includes; for ex. X_CFLAGS
dnl
AC_DEFUN(smr_ARG_WITHINCLUDES, [

AC_ARG_WITH([$1]-includes,
[AC_HELP_STRING([--with-$1-includes=DIR], [set directory for $1 headers])],
[
    if test -d "$withval"; then
        [$1]_CFLAGS="-I${withval}"
    else
        AC_MSG_ERROR(argument must be a directory)
    fi])

    dnl We need to put the given include directory into CPPFLAGS temporarily, but
    dnl then restore CPPFLAGS to its old value.
    dnl
    smr_save_CPPFLAGS="$CPPFLAGS"
    CPPFLAGS="$CPPFLAGS ${[$1]_CFLAGS}"
    ifelse($3, , , CPPFLAGS="$CPPFLAGS [$3]")

    AC_CHECK_HEADERS($2)

    CPPFLAGS=$smr_save_CPPFLAGS
])


dnl Probe for an optional library.  This macro creates both
dnl --with-foo-library and --with-foo-includes options for the configure
dnl script.  If --with-foo-library is *not* specified, the default is to
dnl probe for the library, and use it if found.
dnl
dnl Usage:
dnl smr_CHECK_LIB(name, libname, desc, func, header, x-libs, x-flags)
dnl
dnl name        name for --with options
dnl libname     (optional) real name of library, if different from
dnl             above
dnl desc        (optional) short descr. of library, for help string
dnl func        function of library, to probe for
dnl header      (optional) header required for using library
dnl x-libs      (optional) extra libraries, if needed to link with lib
dnl x-flags     (optional) extra flags, if needed to include header files
dnl
AC_DEFUN(smr_CHECK_LIB,
[
ifelse($2, , smr_lib=[$1], smr_lib=[$2])
ifelse($5, , , smr_header=[$5])
smr_ARG_WITHLIB($1,$2,$3)
if test "$with_$1" != no; then
   AC_CHECK_LIB(ifelse($2, , $1, $2), $4,
        smr_havelib=yes,
        smr_havelib=no; problem_$1=": Can't find working lib$smr_lib",
        ifelse($6, , ${$1_LIBS}, [${$1_LIBS} $6]))
    if test "$smr_havelib" = yes -a "$smr_header" != ""; then
        smr_ARG_WITHINCLUDES($1, $smr_header, $7)
        smr_safe=`echo "$smr_header" | sed 'y%./+-%__p_%'`
        if eval "test \"`echo '$ac_cv_header_'$smr_safe`\" != yes"; then
            smr_havelib=no
            problem_$1=": Can't find working $smr_header"
        fi
    fi
    if test "$smr_havelib" = yes; then
        with_$1=yes
        problem_$1=
    else
        $1_LIBS=
        $1_CFLAGS=
        with_$1=no
    fi
else
    problem_$1=": Explicitly disabled"
fi])


dnl PKG_CHECK_MODULES(GSTUFF, gtk+-2.0 >= 1.3 glib = 1.3.4, action-if, action-not)
dnl defines GSTUFF_LIBS, GSTUFF_CFLAGS, see pkg-config man page
dnl also defines GSTUFF_PKG_ERRORS on error
AC_DEFUN(PKG_CHECK_MODULES, [
  succeeded=no

  if test -z "$PKG_CONFIG"; then
    AC_PATH_PROG(PKG_CONFIG, pkg-config, no)
  fi

  if test "$PKG_CONFIG" = "no" ; then
     echo "*** The pkg-config script could not be found. Make sure it is"
     echo "*** in your path, or set the PKG_CONFIG environment variable"
     echo "*** to the full path to pkg-config."
     echo "*** Or see http://www.freedesktop.org/software/pkgconfig to get pkg-config."
  else
     PKG_CONFIG_MIN_VERSION=0.9.0
     if $PKG_CONFIG --atleast-pkgconfig-version $PKG_CONFIG_MIN_VERSION; then
        AC_MSG_CHECKING(for $2)

        if $PKG_CONFIG --exists "$2" ; then
            AC_MSG_RESULT(yes)
            succeeded=yes

            AC_MSG_CHECKING($1_CFLAGS)
            $1_CFLAGS=`$PKG_CONFIG --cflags "$2"`
            AC_MSG_RESULT($$1_CFLAGS)

            AC_MSG_CHECKING($1_LIBS)
            $1_LIBS=`$PKG_CONFIG --libs "$2"`
            AC_MSG_RESULT($$1_LIBS)
        else
            $1_CFLAGS=""
            $1_LIBS=""
            ## If we have a custom action on failure, don't print errors, but
            ## do set a variable so people can do so.
            $1_PKG_ERRORS=`$PKG_CONFIG --errors-to-stdout --print-errors "$2"`
            ifelse([$4], ,echo $$1_PKG_ERRORS,)
        fi

        AC_SUBST($1_CFLAGS)
        AC_SUBST($1_LIBS)
     else
        echo "*** Your version of pkg-config is too old. You need version $PKG_CONFIG_MIN_VERSION or newer."
        echo "*** See http://www.freedesktop.org/software/pkgconfig"
     fi
  fi

  if test $succeeded = yes; then
     ifelse([$3], , :, [$3])
  else
     ifelse([$4], , AC_MSG_ERROR([Library requirements ($2) not met; consider adjusting the PKG_CONFIG_PATH environment variable if your libraries are in a nonstandard prefix so pkg-config can find them.]), [$4])
  fi
])

dnl Search for a file in the given path.  Search the path from left to right and
dnl set dv_found_path to the found path or "no" if the file was not found.
dnl
dnl Usage:
dnl dv_FIND_FILE(file, path)
dnl
dnl file                name of the file to look for
dnl path                space separated list of directories to search for file
dnl

AC_DEFUN(dv_FIND_FILE,
[
	dv_found_path="no"
	for i in [$2]; do
		if test -f "$i/[$1]"; then
			dv_found_path="$i"
			break;
		fi
	done
])

dnl Check if the include files for a library are accessible, and
dnl define the variable "name_CFLAGS" with the proper "-I" flags for
dnl the compiler.  The user has a chance to specify the includes
dnl location, using "--with-foo-includes".  If the subdir argument is given,
dnl set name_CFLAGS to "-Ipath -Isubdir/path".
dnl
dnl Usage:
dnl dv_ARG_WITHINCLUDES_FIND_HEADER(name, header, subdir, [ACTION-IF-FOUND],
dnl 	[ACTION-IF-NOT-FOUND], [INCLUDES = `default-includes'], search-path)
dnl
dnl name                library name, MUST same as used with smr_ARG_WITHLIB
dnl header              a header file required for using the lib
dnl subdir              header resides in subdir below include path
dnl ACTION-IF-FOUND     commands to execute if file is found
dnl ACTION-IF-NOT-FOUND commands to execute if file is not found
dnl INCLUDES            passed to AC_CHECK_HEADER macro
dnl search-path         default search path
dnl

AC_DEFUN(dv_ARG_WITHINCLUDES_FIND_HEADER,
[
	ifelse([$7], , dv_default_path="/usr/include /usr/local/include",
		dv_default_path="[$7]")
	AC_ARG_WITH(
		[$1]-includes,
		[AC_HELP_STRING(
			[--with-$1-includes=DIR],
			[set directory for $1 headers])],
		[
			if test -d "$withval"; then
				dv_search_path="${withval}"
			else
				AC_MSG_ERROR(argument must be a directory)
			fi
		],
		dv_search_path="${dv_default_path}"
	)
	ifelse($3, , dv_header="[$2]", dv_header="[$3]/[$2]")
	AC_MSG_CHECKING(for path to [$dv_header])
	dv_FIND_FILE([$dv_header], [$dv_search_path])
	AC_MSG_RESULT([$dv_found_path])
 	if test x"[$dv_found_path]" = xno; then
 		[$5]
 	else
		dv_flags="-I${dv_found_path}"
		for i in ${dv_default_path}; do
			test x"$i" = x"$dv_found_path" && dv_flags=""
		done
		ifelse($3, , , dv_flags="${dv_flags} -I${dv_found_path}/[$3]")
		dv_save="${CPPFLAGS}"
		CPPFLAGS="${CPPFLAGS} ${dv_flags}"
		AC_CHECK_HEADER(
			[${dv_found_path}/${dv_header}], [$4], [$5], [$6])
		[$1]_CFLAGS="${dv_flags}"
		CPPFLAGS="${dv_save}"
 	fi
])

dnl Probe for an optional library.  This macro creates both
dnl --with-foo-library and --with-foo-includes options for the configure
dnl script.  If --with-foo-library is *not* specified, the default is to
dnl probe for the library, and use it if found.
dnl
dnl Usage:
dnl dv_CHECK_LIB(name, libname, desc, func, header, x-libs, x-flags)
dnl
dnl name        name for --with options
dnl libname     (optional) real name of library, if different from
dnl             above
dnl desc        (optional) short descr. of library, for help string
dnl func        function of library, to probe for
dnl x-libs      (optional) extra libraries, if needed to link with lib
dnl x-flags     (optional) extra flags, if needed to include header files
dnl
AC_DEFUN(dv_CHECK_LIB,
[
	ifelse($2, , dv_lib=[$1], dv_lib=[$2])
	smr_ARG_WITHLIB($1,$2,$3)
	if test "$with_$1" != no; then
		AC_CHECK_LIB(ifelse($2, , $1, $2), $4,
			dv_havelib=yes,
			dv_havelib=no;
			problem_$1=": Can't find working lib$dv_lib",
			ifelse($5, , ${$1_LIBS}, [${$1_LIBS} $5])
		)
		if test x"$dv_havelib" = xyes; then
			with_$1=yes
			problem_$1=
		else
			$1_LIBS=
			with_$1=no
		fi
	else
		problem_$1=": Explicitly disabled"
	fi
])

dnl Just empties and exports the dc_configure_flags variable.  Useful in
dnl combination with the dnl macros below.
dnl
dnl Usage:
dnl dv_INIT_DC_CONFFLAGS
dnl
AC_DEFUN(dv_INIT_DC_CONFFLAGS, [
	dc_configure_flags=""
	AC_SUBST(dc_configure_flags)
])

dnl Add a flag to the dc_configure_flags variable to be passes to the
dnl DISTCHECK_CONFIGURE_FLAGS automake macro later.
dnl
dnl Usage:
dnl dv_ADD_DC_CONFFLAG(option)
dnl
dnl option      the string to be appended to dc_configure_flags
dnl
AC_DEFUN(dv_ADD_DC_CONFFLAG, [dc_configure_flags="$dc_configure_flags [$1]"])

dnl Add an "option=value" pair to the dc_configure_flags variable to be passes
dnl to the DISTCHECK_CONFIGURE_FLAGS automake macro later.  The option is added
dnl only if the passed value is not empty and not the literal string "yes".
dnl
dnl Usage:
dnl dv_ADD_DC_CONFFLAG_WITH_VALUE(option, value, force)
dnl
dnl option      the name of the option to be appended to dc_configure_flags
dnl value       value that is appended as "option=value"
dnl force       if this value is not empty, force setting the value even if it
dnl             is "yes". (optional)
dnl
AC_DEFUN(dv_ADD_DC_CONFFLAG_WITH_VALUE, [
	test -n "[$2]" && test x"[$2]" != x"yes" -o x"[$3]" != x &&
	dc_configure_flags="$dc_configure_flags [$1]=[$2]"
])

AC_DEFUN(dv_MSG_ERROR_MISSING, [
	if test x"$3" = x -o x"$3" = xno; then
		ifelse($2, , dv_msg_error_addtext="",
			dv_msg_error_addtext=". [$2]")
		AC_MSG_ERROR([$1]$dv_msg_error_addtext)
	fi
])

dnl Some small macros that print a standard error message for a missing header
dnl file or library.
dnl
dnl Usage:
dnl dv_MSG_ERROR_MISSING_HEADER(file, text, [NOVAL])
dnl dv_MSG_ERROR_MISSING_LIBRARY(library, text, [NOVAL])
dnl dv_MSG_ERROR_MISSING_PROG(program, text, [NOVAL])
dnl
dnl file        name or path of the missing header file
dnl library     name of the missing library
dnl program     name of the missing program
dnl text        additional text to be printed in the error message
dnl NOVAL       (optional) if given, print the message only if NOVAL is "no"
dnl

AC_DEFUN(dv_MSG_ERROR_MISSING_HEADER, [
	dv_MSG_ERROR_MISSING([failed to detect header file $1],[$2],[$3])
])
AC_DEFUN(dv_MSG_ERROR_MISSING_LIBRARY, [
	dv_MSG_ERROR_MISSING([failed to detect [$1] library],[$2],[$3])
])
AC_DEFUN(dv_MSG_ERROR_MISSING_PROG, [
	dv_MSG_ERROR_MISSING([program $1 not found in PATH],[$2],[$3])
])

dnl Some small macros that print a standard string that asks to install a
dnl package.
dnl
dnl Usage:
dnl dv_MSG_INSTALL_DEV_PACKAGE_STRING(library)
dnl dv_MSG_INSTALL_PACKAGE_STRING(package)
dnl
dnl library      name of library of which the header files could not be found
dnl package      missing package
dnl

AC_DEFUN(dv_MSG_INSTALL_DEV_PACKAGE_STRING,
	[Please install the $1 development package.]
)

AC_DEFUN(dv_MSG_INSTALL_PACKAGE_STRING, [Please install the $1 package.])

dnl Macro that generates a number of environment variables from @VERSION@ for
dnl use with the template version script and sources:
dnl
dnl V_MAJOR, V_MINOR, V_PATCH, V_NUMBER
dnl LV_AGE, LV_IF, LV_REV, LV_NUMBER
dnl
dnl Usage:
dnl dv_MAKE_VERSION_NUMBERS
dnl
AC_DEFUN(dv_MAKE_VERSION_NUMBERS, [
	V_MAJOR=`echo ${VERSION} | "$SED" -e 's/\..*//'`
	V_MINOR=`echo ${VERSION} | \
		"$SED" -e 's/^[[0123456789]]*\.//' -e 's/\..*//'`
	V_PATCH=`echo ${VERSION} | \
		"$SED" -e 's/.*\.//' -e 's/[[^0123456789]]*$//'`
	V_NUMBER=$[[V_MAJOR * 16777216 + V_MINOR * 65536 + V_PATCH]]
	LV_AGE=`echo ${SO_VERSION} | "$SED" -e 's/:.*//'`
	LV_IF=`echo ${SO_VERSION} | \
		"$SED" -e 's/^[[0123456789]]*://' -e 's/:.*//'`
	LV_REV=`echo ${SO_VERSION} | "$SED" -e 's/.*://'`
	LV_NUMBER=$[[LV_AGE * 16777216 + LV_IF * 65536 + LV_REV]]
	AC_SUBST(V_MAJOR)
	AC_SUBST(V_MINOR)
	AC_SUBST(V_PATCH)
	AC_SUBST(V_NUMBER)
	AC_SUBST(LV_AGE)
	AC_SUBST(LV_IF)
	AC_SUBST(LV_REV)
	AC_SUBST(LV_NUMBER)
])

dnl Macro checking for sp library version scripts and a specific library
dnl version or newer.
dnl
dnl Usage:
dnl dv_CHECK_TEMPLATE_LIB_VERSION(key, lib, MAJOR,MINOR,PATCH, LAGE,LIF,LREV)
dnl
dnl key         unique key for this test
dnl lib         name of the library, script name is <lib>-version
dnl MAJOR       (optional) required major version number of library
dnl MINOR       (optional) required minor version number of library
dnl PATCH       (optional) required patchlevel of library
dnl LAGE        (optional) required major version number of library interface
dnl LIF         (optional) required minor version number of library interface
dnl LREV        (optional) required patchlevel of library interface
dnl
dnl All version numbers default to 0 if not specified.
dnl
AC_DEFUN(dv_CHECK_TEMPLATE_LIB_VERSION, [
	ifelse($3, , V1=0, V1=$3)
	ifelse($4, , V2=0, V2=$4)
	ifelse($5, , V3=0, V3=$5)
	ifelse($6, , L1=0, L1=$6)
	ifelse($7, , L2=0, L2=$7)
	ifelse($8, , L3=0, L3=$8)
	AC_PATH_PROG([$1], [$2-version])
	test x"$[$1]" = x && AC_MSG_ERROR([$2-version not found in PATH])
	AC_MSG_CHECKING([lib$2 version $V1.$V2.$V3, $L1:$L2:$L3 or later])
	LIB_V_PACKAGE=""
	LIB_LV_PACKAGE=""
	eval `[$2-version]`
	test x"$LIB_V_PACKAGE" = x[$2] ||
		AC_MSG_ERROR([$2-version found in PATH does not work])
	dv_VN=$[[V1 * 16777216 + V2 * 65536 + V3]]
	dv_LVN=$[[L1 * 16777216 + L2 * 65536 + L3]]
	test x"$[[LIB_V_NUMBER < dv_VN]]" = x1 &&
		AC_MSG_ERROR([lib$2 is too old ($V1.$V2.$V3 is required)])
	test x"$[[LIB_LV_NUMBER < dv_LVN]]" = x1 &&
		AC_MSG_ERROR([lib$2 is too old ($L1:$L2:$L3 is required)])
	AC_MSG_RESULT([$LIB_V_MAJOR.$LIB_V_MINOR.$LIB_V_PATCH, $LIB_LV_AGE:$LIB_LV_IF:$LIB_LV_REV])
])
