/* gateguardian_if.h */

/* ---------------------------- interface types ---------------------------- */

typedef enum
{
	GATEG_EVKEEP = 0,
	/* filters */
	GATEG_EVKEEP_REJECT_PATH,
	GATEG_EVKEEP_FILTER_LANG,
	GATEG_EVKEEP_FILTER_INT,
	GATEG_EVKEEP_FILTER_TERM,

	GATEG_EVSET,
	/* special values */
	GATEG_EVSET_USER_REAL,
	GATEG_EVSET_LOGINDIR_REAL,
	GATEG_EVSET_USER_EFFECTIVE,
	GATEG_EVSET_LOGINDIR_EFFECTIVE,

	/* do not touch these */
	GATEG_EVEOT = -1,
	GATEG_EVSKIP = -2,
	GATEG_EVDEL = -3
} gateg_evaction_t;

typedef struct
{
	char *e_var;
	char *e_val;
	gateg_evaction_t e_action;
	/* private, don't touch this */
	const int e_is_handled;
} gateg_evarhandler_t;

typedef enum
{
	GATEG_OP_DFLT = 0x0,
	GATEG_OP_SET = 0x1,
	GATEG_OP_NOP = 0x2,
	GATEG_OP_PARANOID = 0x4,
	GATEG_OP_DFLT_PARANOID = GATEG_OP_DFLT | GATEG_OP_PARANOID,
	GATEG_OP_SET_PARANOID = GATEG_OP_SET | GATEG_OP_PARANOID,
	/* add to one of the above basic modes */
	GATEG_OP_DELAY_1 = 0x1000,
	GATEG_OP_DELAY_2 = 0x2000,
	GATEG_OP_DELAY_3 = 0x3000,
	GATEG_OP_DELAY_4 = 0x4000,
	GATEG_OP_DELAY_5 = 0x5000,
	GATEG_OP_DELAY_6 = 0x6000,
	GATEG_OP_DELAY_7 = 0x7000,
	GATEG_OP_DELAY_8 = 0x8000,
	GATEG_OP_DELAY_9 = 0x9000,
	GATEG_OP_DELAY_10 = 0xa000,
	GATEG_OP_DELAY_11 = 0xb000,
	GATEG_OP_DELAY_12 = 0xc000,
	GATEG_OP_DELAY_13 = 0xd000,
	GATEG_OP_DELAY_14 = 0xe000,
	GATEG_OP_DELAY_15 = 0xf000,
	/* for internal purposes only */
	GATEG_OP_MASK = 0x3,
	GATEG_OP_DELAY_MASK = 0xf000,
	GATEG_OP_DONE = 0x800
} gateg_op_t;

typedef enum
{
	GATEG_INIT_DFLT,
	GATEG_INIT_DFLT_PARANOID,
	GATEG_INIT_NOP
	/*,!!!GATEG_INIT_DAEMON,*/
	/*GATEG_INIT_CLIENT,*/
	/*GATEG_INIT_SUID,*/
	/*GATEG_INIT_FORK,*/
	/*GATEG_INIT_EXEC,*/
} gateg_init_t;

typedef struct
{
	struct
	{
		unsigned int evars_clean;
		unsigned int fds_close;
		unsigned int fds_std;
		unsigned int cwd_set;
		unsigned int chroot_call;
		unsigned int chroot_verify;
		unsigned int umask_set;
		unsigned int priv_set;
		unsigned int priv_drop;
		unsigned int sighand_reset;
		unsigned int sighand_verify;
		unsigned int core_disable;
	} ops;
	gateg_evarhandler_t *evars_handlers;
	char *cwd;
	char *chroot_dir;
	mode_t umask;
	uid_t priv_uid;
	gid_t priv_gid;
	/* private, don't touch this */
	unsigned int identifier_token;
} gateg_cfg_t;

/* ---------------------------- forward declarations ----------------------- */

void gateg_init_cfg(gateg_cfg_t *argp, gateg_init_t in);
int gateg_safe_init(gateg_cfg_t *scfg);
int gateg_chroot(const char *path);
int gateg_safe_run_command(char * const argv[]);
int gateg_safe_run_command_with_cfg(gateg_cfg_t *cfg, char * const argv[]);
