/* -*-c-*- */

/* ---------------------------- included header files ---------------------- */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/* ---------------------------- local definitions -------------------------- */

/* for debugging this test program, set it to 1 */
#define GATEG_TEST_DEBUG 1

#define GATEG_DEBUG

/* ---------------------------- included code files ------------------------ */

#include "inputguardian.c"

/* ---------------------------- feature tests ------------------------------ */

/* ---------------------------- local macros ------------------------------- */

/* Aborts if the called code does not call abort() and vice versa.
 * This can be written only as a macro because it takes a piece of code as an
 * argument.
 */
#define ASSERT_ABORT(code) \
	do \
	{ \
		if (setjmp(__gateg_debug_jmp_buf) == 0) \
		{ \
			do { code; } while (0); \
			/* If we get here, longjmp was never called, */ \
			/* i.e. the assertion failed */ \
			abort(); \
		} \
		/* code triggered an abort(), ok */ \
	} while (0)

/* ---------------------------- imports ------------------------------------ */

/* ---------------------------- local types -------------------------------- */

typedef int (*test_func_t)(void);
typedef char *(*html_test_func_t)(const char *input, ...);

typedef struct
{
	test_func_t t_func;
	const char *t_name;
	const char *t_msg;
	int t_resulttype;
} test_table_entry_t;

typedef struct
{
	const char *t_input;
	const char *t_escaped;
} html_test_table_entry_t;

typedef struct
{
	const char *t_addr;
	int t_expected_rc;
} email_test_table_entry_t;

enum
{
	RET_OK = 0,
	/* errors */
	RET_FAILED = 255
};

/* ---------------------------- local variables ---------------------------- */

static html_test_table_entry_t html_test_table1[] =
{
	{ "", "" },
	{
		"qwertasdfgyxcvbpoiuzlkjhmn0987123456",
		"qwertasdfgyxcvbpoiuzlkjhmn0987123456"
	},
	{ "&\"><", "&amp;&quot;&gt;&lt;" },
	{ "<>", "&lt;&gt;" },
	{ "<x>", "&lt;x&gt;" },
	{ "<abbr>", "<abbr>" },
	{ "<acronym>", "<acronym>" },
	{ "<b>", "<b>" },
	{ "<bdo>", "<bdo>" },
	{ "<big>", "<big>" },
	{ "<blink>", "<blink>" },
	{ "<blockquote>", "<blockquote>" },
	{ "<br>", "<br>" },
	{ "<center>", "<center>" },
	{ "<cite>", "<cite>" },
	{ "<code>", "<code>" },
	{ "<dd>", "<dd>" },
	{ "<del>", "<del>" },
	{ "<dfn>", "<dfn>" },
	{ "<dir>", "<dir>" },
	{ "<dl>", "<dl>" },
	{ "<dt>", "<dt>" },
	{ "<em>", "<em>" },
	{ "<h1>", "<h1>" },
	{ "<h2>", "<h2>" },
	{ "<h3>", "<h3>" },
	{ "<h4>", "<h4>" },
	{ "<h5>", "<h5>" },
	{ "<h6>", "<h6>" },
	{ "<hr>", "<hr>" },
	{ "<i>", "<i>" },
	{ "<ins>", "<ins>" },
	{ "<kbd>", "<kbd>" },
	{ "<li>", "<li>" },
	{ "<menu>", "<menu>" },
	{ "<nobr>", "<nobr>" },
	{ "<ol>", "<ol>" },
	{ "<p>", "<p>" },
	{ "<plaintext>", "<plaintext>" },
	{ "<pre>", "<pre>" },
	{ "<q>", "<q>" },
	{ "<s>", "<s>" },
	{ "<samp>", "<samp>" },
	{ "<small>", "<small>" },
	{ "<spacer>", "<spacer>" },
	{ "<strike>", "<strike>" },
	{ "<strong>", "<strong>" },
	{ "<sub>", "<sub>" },
	{ "<sup>", "<sup>" },
	{ "<tt>", "<tt>" },
	{ "<u>", "<u>" },
	{ "<ul>", "<ul>" },
	{ "<var>", "<var>" },
	{ "<br><br><br><br><br>", "<br><br><br><br><br>" },
	{ "&lt;", "&amp;lt;" },
	{ "<a href=\"http://x.y/z\">", "<a href=\"http://x.y/z\">" },
	{ "<a href=\"http://x\">", "<a href=\"http://x\">" },
	{ "<a  href=\"http://x\">", "<a  href=\"http://x\">" },
	{ "<a href =\"http://x\">", "<a href =\"http://x\">" },
	{ "<a href= \"http://x\">", "<a href= \"http://x\">" },
	{ "<a href=http://x>", "<a href=http://x>" },
	{ "<a href=\"http://x>", "&lt;a href=&quot;http://x&gt;" },
	{ "<a href=http://x?>", "&lt;a href=http://x?&gt;" },
	{ "<a href=http://x#>", "&lt;a href=http://x#&gt;" },
	{ "<a href=http://x&>", "&lt;a href=http://x&amp;&gt;" },
	{ (char *)-1, (char *)-1 }
};

static html_test_table_entry_t html_test_table2[] =
{
	{ "", "" },
	{
		"qwertasdfgyxcvbpoiuzlkjhmn0987123456",
		"qwertasdfgyxcvbpoiuzlkjhmn0987123456"
	},
	{ "&\"><", "&amp;&quot;&gt;&lt;" },
	{ "<>", "&lt;&gt;" },
	{ "<x>", "&lt;x&gt;" },
	{ "<abbr>", "&lt;abbr&gt;" },
	{ "<acronym>", "&lt;acronym&gt;" },
	{ "<b>", "&lt;b&gt;" },
	{ "<bdo>", "&lt;bdo&gt;" },
	{ "<big>", "&lt;big&gt;" },
	{ "<blink>", "&lt;blink&gt;" },
	{ "<blockquote>", "&lt;blockquote&gt;" },
	{ "<br>", "&lt;br&gt;" },
	{ "<center>", "&lt;center&gt;" },
	{ "<cite>", "&lt;cite&gt;" },
	{ "<code>", "&lt;code&gt;" },
	{ "<dd>", "&lt;dd&gt;" },
	{ "<del>", "&lt;del&gt;" },
	{ "<dfn>", "&lt;dfn&gt;" },
	{ "<dir>", "&lt;dir&gt;" },
	{ "<dl>", "&lt;dl&gt;" },
	{ "<dt>", "&lt;dt&gt;" },
	{ "<em>", "&lt;em&gt;" },
	{ "<h1>", "&lt;h1&gt;" },
	{ "<h2>", "&lt;h2&gt;" },
	{ "<h3>", "&lt;h3&gt;" },
	{ "<h4>", "&lt;h4&gt;" },
	{ "<h5>", "&lt;h5&gt;" },
	{ "<h6>", "&lt;h6&gt;" },
	{ "<hr>", "&lt;hr&gt;" },
	{ "<i>", "&lt;i&gt;" },
	{ "<ins>", "&lt;ins&gt;" },
	{ "<kbd>", "&lt;kbd&gt;" },
	{ "<li>", "&lt;li&gt;" },
	{ "<menu>", "&lt;menu&gt;" },
	{ "<nobr>", "&lt;nobr&gt;" },
	{ "<ol>", "&lt;ol&gt;" },
	{ "<p>", "&lt;p&gt;" },
	{ "<plaintext>", "&lt;plaintext&gt;" },
	{ "<pre>", "&lt;pre&gt;" },
	{ "<q>", "&lt;q&gt;" },
	{ "<s>", "&lt;s&gt;" },
	{ "<samp>", "&lt;samp&gt;" },
	{ "<small>", "&lt;small&gt;" },
	{ "<spacer>", "&lt;spacer&gt;" },
	{ "<strike>", "&lt;strike&gt;" },
	{ "<strong>", "&lt;strong&gt;" },
	{ "<sub>", "&lt;sub&gt;" },
	{ "<sup>", "&lt;sup&gt;" },
	{ "<tt>", "&lt;tt&gt;" },
	{ "<u>", "&lt;u&gt;" },
	{ "<ul>", "&lt;ul&gt;" },
	{ "<var>", "&lt;var&gt;" },
	{
		"<br><br><br><br><br>",
		"&lt;br&gt;&lt;br&gt;&lt;br&gt;&lt;br&gt;&lt;br&gt;"
	},
	{ "&lt;", "&amp;lt;" },
	{
		"<a href=\"http://x.y/z\">",
		"&lt;a href=&quot;http://x.y/z&quot;&gt;"
	},
	{ "<a href=\"http://x\">", "&lt;a href=&quot;http://x&quot;&gt;" },
	{ "<a  href=\"http://x\">", "&lt;a  href=&quot;http://x&quot;&gt;" },
	{ "<a href =\"http://x\">", "&lt;a href =&quot;http://x&quot;&gt;" },
	{ "<a href= \"http://x\">", "&lt;a href= &quot;http://x&quot;&gt;" },
	{ "<a href=http://x>", "&lt;a href=http://x&gt;" },
	{ "<a href=\"http://x>", "&lt;a href=&quot;http://x&gt;" },
	{ "<a href=http://x?>", "&lt;a href=http://x?&gt;" },
	{ "<a href=http://x#>", "&lt;a href=http://x#&gt;" },
	{ "<a href=http://x&>", "&lt;a href=http://x&amp;&gt;" },
	{ (char *)-1, (char *)-1 }
};

static email_test_table_entry_t email_test_table[] =
{
	{ "", 0 },
	{ "@", 0 },
	{ "foo", 0 },
	{ "foo@", 0 },
	{ "@bar", 0 },
	{ "a@b.", 0 },
	{ "a@b", 1 },
	{ "a.a@b.b", 1 },
	{ "a@b.c.d.e.f.g.h.i.j.k.l.m.n.o.p.q.r.s.t.u.v.w.x.y.z", 1 },
	{
		"abcdefghijklmnopqrstuvwxyz"
		"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
		"0123456789+-_."
		"@"
		"abcdefghijklmnopqrstuvwxyz"
		"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
		"0123456789-",
		1
	},
	{ ",@b", 0 },
	{ "b@,", 0 },
	{ "@@b", 0 },
	{ "a@@", 0 },
	{ ".a@-a.-a", 1 },
	{ "a.@a-.a-", 1 },
	{ 0, 0 }
};

/* ---------------------------- local functions ---------------------------- */

static int __str_equal(const char *s1, const char *s2)
{
	if (s1 == 0)
	{
		return (s2 == 0);
	}
	for ( ; *s1 != 0 && *s1 == *s2; s1++, s2++)
	{
		/* nothing */
	}
	if (*s1 == 0 && *s2 == 0)
	{
		return 1;
	}

	return 0;
}

/* ---------------------------- test functions ----------------------------- */

static int __test_i(void)
{
	const char *end;
	int rc;
	int ival;

	rc = inputg_parse_i("1234567890", &end, &ival);
	if (rc != 0 || ival != 1234567890 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_i("-1234567890", &end, &ival);
	if (rc != 0 || ival != -1234567890 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_i(
		"1234567890123456789012345678901234567890", &end, &ival);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}

	return RET_OK;
}

static int __test_ui(void)
{
	const char *end;
	int rc;
	unsigned int uival;

	rc = inputg_parse_ui("1234567890", &end, &uival);
	if (rc != 0 || uival != 1234567890 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_ui("-1234567890", &end, &uival);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_ui(
		"1234567890123456789012345678901234567890", &end, &uival);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}

	return RET_OK;
}

static int __test_l(void)
{
	const char *end;
	int rc;
	long int lval;

	rc = inputg_parse_l("1234567890", &end, &lval);
	if (rc != 0 || lval != 1234567890 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_l("-1234567890", &end, &lval);
	if (rc != 0 || lval != -1234567890 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_l(
		"1234567890123456789012345678901234567890", &end, &lval);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}

	return RET_OK;
}

static int __test_ul(void)
{
	const char *end;
	int rc;
	unsigned long int ulval;

	rc = inputg_parse_ul("1234567890", &end, &ulval);
	if (rc != 0 || ulval != 1234567890 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_ul("-1234567890", &end, &ulval);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_ul(
		"1234567890123456789012345678901234567890", &end, &ulval);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}

	return RET_OK;
}

static int __test_s(void)
{
	const char *end;
	int rc;
	short int sval;

	rc = inputg_parse_s("32767", &end, &sval);
	if (rc != 0 || sval != 32767 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_s("32768", &end, &sval);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_s("-32768", &end, &sval);
	if (rc != 0 || sval != -32768 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_s("-32769", &end, &sval);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}

	return RET_OK;
}

static int __test_us(void)
{
	const char *end;
	int rc;
	unsigned short int usval;

	rc = inputg_parse_us("0", &end, &usval);
	if (rc != 0 || usval != 0 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_us("-1", &end, &usval);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_us("65535", &end, &usval);
	if (rc != 0 || usval != 65535 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_us("65536", &end, &usval);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}

	return RET_OK;
}

static int __test_sc(void)
{
	const char *end;
	int rc;
	signed char cval;

	rc = inputg_parse_sc("127", &end, &cval);
	if (rc != 0 || cval != 127 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_sc("128", &end, &cval);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_sc("-128", &end, &cval);
	if (rc != 0 || cval != -128 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_sc("-129", &end, &cval);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}

	return RET_OK;
}

static int __test_uc(void)
{
	const char *end;
	int rc;
	unsigned char ucval;

	rc = inputg_parse_uc("0", &end, &ucval);
	if (rc != 0 || ucval != 0 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_uc("-1", &end, &ucval);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_uc("255", &end, &ucval);
	if (rc != 0 || ucval != 255 || *end != 0)
	{
		return RET_FAILED;
	}
	rc = inputg_parse_uc("256", &end, &ucval);
	if (rc == 0 || *end != 0)
	{
		return RET_FAILED;
	}

	return RET_OK;
}

static int __test_html(
	html_test_table_entry_t *table, html_test_func_t func,
	const void *args)
{
	int rc;
	int i;

	for (
		i = 0, rc = RET_OK;
		table[i].t_input != (char *)-1 && rc == RET_OK; i++)
	{
		char *ret;

		if (
			func ==
			(html_test_func_t)inputg_escape_html_with_tag_table)
		{
			ret = func(table[i].t_input, args);
		}
		else
		{
			ret = func(table[i].t_input);
		}
		if (ret == 0)
		{
			fprintf(stderr, "  test failed: out of memory\n");
			fprintf(
				stderr, "    input   : '%s'\n",
				table[i].t_input);
			rc = RET_FAILED;
		}
		else
		{
			int is_equal;

			is_equal = __str_equal(ret, table[i].t_escaped);
			if (is_equal == 0)
			{
				rc = RET_FAILED;
				fprintf(stderr, "  test failed\n");
				fprintf(
					stderr,
					"    input   : '%s'\n",
					table[i].t_input);
				fprintf(stderr, "    output  : '%s'\n", ret);
				fprintf(
					stderr,
					"    expected: '%s'\n",
					table[i].t_escaped);
			}
			free(ret);
		}
	}

	return rc;
}

static int __test_html1(void)
{
	int rc;

	rc = __test_html(
		html_test_table1, (html_test_func_t)inputg_escape_html, 0);

	return rc;
}

static int __test_html2(void)
{
	int rc;

	rc = __test_html(
		html_test_table2, (html_test_func_t)inputg_escape_all_html, 0);

	return rc;
}

static int __test_html3(void)
{
	int rc;

	rc = __test_html(
		html_test_table1,
		(html_test_func_t)inputg_escape_html_with_tag_table,
		__inputg_allowed_formatters);
	if (rc != RET_OK)
	{
		return rc;
	}
	rc = __test_html(
		html_test_table2,
		(html_test_func_t)inputg_escape_html_with_tag_table, 0);

	return rc;
}

static int __test_email(void)
{
	int rc;
	int i;

	rc = RET_OK;
	for (i = 0, rc = RET_OK; email_test_table[i].t_addr != 0; i++)
	{
		int ret;

		ret = inputg_is_simple_email_address(
			email_test_table[i].t_addr);
		if (ret != email_test_table[i].t_expected_rc)
		{
			rc = RET_FAILED;
			fprintf(stderr, "  test failed\n");
			fprintf(
				stderr, "    rc %d(%d), addr: '%s'\n", ret,
				email_test_table[i].t_expected_rc,
				email_test_table[i].t_addr);
		}
	}

	return rc;
}

/* ---------------------------- local variables ---------------------------- */

static test_table_entry_t test_table[] =
{
	{ __test_i, "int", "test inputg_parse_i", RET_OK },
	{ __test_ui, "uint", "test inputg_parse_ui", RET_OK },
	{ __test_l, "long int", "test inputg_parse_l", RET_OK },
	{ __test_ul, "long uint", "test inputg_parse_ul", RET_OK },
	{ __test_s, "short int", "test inputg_parse_s", RET_OK },
	{ __test_us, "short uint", "test inputg_parse_us", RET_OK },
	{ __test_sc, "signed char", "test inputg_parse_sc", RET_OK },
	{ __test_uc, "unsigned char", "test inputg_parse_uc", RET_OK },
	{ __test_html1, "html1", "test inputg_escape_html", RET_OK },
	{ __test_html2, "html2", "test inputg_escape_all_html", RET_OK },
	{
		__test_html3,
		"html3",
		"test inputg_escape_html_with_tag_table",
		RET_OK
	},
	{
		__test_email,
		"email",
		"test inputg_is_simple_email_address",
		RET_OK
	},
	{ 0, 0, 0, RET_OK }
};

/* ---------------------------- local functions ---------------------------- */

static int __run_test(test_table_entry_t *test)
{
	pid_t child = 0;
	pid_t pid = 0;
	int status = 0;
	int rc;

	child = 0;
	pid = 0;
	status = 0;
	fprintf(stderr, "*** executing test %s\n", test->t_name);
	if (test->t_msg != 0)
	{
		fprintf(stderr, "    %s\n", test->t_msg);
	}
	if (GATEG_TEST_DEBUG == 0)
	{
		child = fork();
		switch (child)
		{
		case (pid_t)-1:
			perror("fork");
			abort();
		case 0:
			/* child */
			rc = test->t_func();
			exit(rc);
		default:
			/* parent */
			pid = wait(&status);
			rc = WEXITSTATUS(status);
			break;
		}
	}
	else
	{
		rc = test->t_func();
	}
	switch (rc)
	{
	case RET_OK:
		fprintf(stderr, "OK\n");
		break;
	case RET_FAILED:
	default:
		fprintf(stderr, "FAILED with return code %d\n", rc);
		break;
	}

	return rc;
}

/* ---------------------------- interface functions ------------------------ */

int main(void)
{
	int tests_failed;
	int rc;
	int i;

	fprintf(
		stderr, "If this program aborts, there is a flaw in the"
		" tested code\n"
		"(Aborted sub-processes are part of the tests.)\n");
	tests_failed = 0;
	for (i = 0; test_table[i].t_func != 0; i++)
	{
		rc = __run_test(&test_table[i]);
		switch (rc)
		{
		case RET_FAILED:
			tests_failed++;
			break;
		case RET_OK:
			break;
		default:
			fprintf(stderr, "\nERROR: invalid return code\n");
			abort();
			exit(1);
			break;
		}
	}
	if (tests_failed != 0)
	{
		fprintf(stderr, "\nERROR: at least one test has failed\n");

		return 1;
	}

	return 0;
}
